/* ----------------------------------------------------------
(c) 2007 Sylvain Lefebvre - all rights reserved
--
The source code is provided 'as it is', without any warranties. 
Use at your own risk. The use of any part of the source code in a
commercial or non commercial product without explicit authorisation
from the author is forbidden. Use for research and educational
purposes is allowed and encouraged, provided that a short notice
acknowledges the author's work.
---------------------------------------------------------- */
//---------------------------------------------------------------------------

#include "Image.h"

#include <algorithm>

//---------------------------------------------------------------------------

#define NAMESPACE 

//---------------------------------------------------------------------------

// image format manager unique instance
NAMESPACE::ImageFormatManager *NAMESPACE::ImageFormatManager::s_Manager = NULL;

//---------------------------------------------------------------------------

NAMESPACE::ImageFormatManager *NAMESPACE::ImageFormatManager::getUniqueInstance()
{
  if (s_Manager==NULL) {
    s_Manager=new ImageFormatManager();
  }
  return (s_Manager);
}

//---------------------------------------------------------------------------

// #include <LibSL/Image/ImageFormat_PNG.h>

NAMESPACE::ImageFormatManager::ImageFormatManager()
{
  //registerPlugin(new NAMESPACE::ImageFormat_PNG()); // handled by an auto pointer  FIXME: only with VC++
}

//---------------------------------------------------------------------------

void NAMESPACE::ImageFormatManager::registerPlugin(ImageFormat_plugin *plugin)
{
  std::map<std::string,ImageFormat_plugin *>::iterator 
    P=m_Plugins.find(std::string(plugin->signature()));
  if (P == m_Plugins.end()) {
    m_Plugins[std::string(plugin->signature())]=plugin;
    // std::cerr << sprint("[ImageFormatManager] registered %s format.\n",plugin->signature());
  } else {
    // LIBSL_FATAL_ERROR_WITH_ARGS("ImageFormatManager::registerPlugin - plugin '%s' is already present !",plugin->signature());
  }
}

//---------------------------------------------------------------------------

NAMESPACE::ImageFormat_plugin *
NAMESPACE::ImageFormatManager::getPlugin(const char *signature) const
{
  std::string ext=std::string(signature);
  std::transform(ext.begin(),ext.end(), ext.begin(), tolower);
   std::map<std::string,NAMESPACE::ImageFormat_plugin *>::const_iterator 
    P=m_Plugins.find(ext);
  if (P == m_Plugins.end()) {
    fprintf(stderr,"ImageFormatManager - Cannot find any plugin for '%s' (unknown format)",signature);
    exit (-1);
  } else {
    return ((*P).second);
  }
}

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------

NAMESPACE::Image *NAMESPACE::loadImage_raw(const char *fname)
{
  NAMESPACE::ImageFormatManager& 
    manager=(*NAMESPACE::ImageFormatManager::getUniqueInstance());
  const char *pos=strrchr(fname,'.');
  if (pos == NULL) {
    fprintf(stderr,"Image - Cannot determine file type ('%s')",fname);
    exit (-1);
  }
  return (manager.getPlugin(pos+1)->load(fname));
}

//---------------------------------------------------------------------------
