/* ----------------------------------------------------------
(c) 2007 Sylvain Lefebvre - all rights reserved
--
The source code is provided 'as it is', without any warranties. 
Use at your own risk. The use of any part of the source code in a
commercial or non commercial product without explicit authorisation
from the author is forbidden. Use for research and educational
purposes is allowed and encouraged, provided that a short notice
acknowledges the author's work.
---------------------------------------------------------- */
//---------------------------------------------------------------------------

#include "ImageFormat_PNG.h"
#include <stdio.h>

#include <vector>
using namespace std;

//---------------------------------------------------------------------------

extern "C" {
#include "png.h"
}

//---------------------------------------------------------------------------

#define NAMESPACE

//---------------------------------------------------------------------------

NAMESPACE::ImageFormat_PNG::ImageFormat_PNG()
{
  IMAGE_FORMAT_MANAGER.registerPlugin(this);
}

//---------------------------------------------------------------------------

NAMESPACE::Image *NAMESPACE::ImageFormat_PNG::load(const char *name) const
{
  FILE *file;
  fopen_s(&file,name,"rb");

  if (file == NULL) {
    fprintf(stderr,"ImageFormat_PNG::load - cannot open %s",name);
    exit (-1);
  }

  png_structp png_ptr =
    png_create_read_struct(PNG_LIBPNG_VER_STRING,
    0, // (png_voidp)user_error_ptr
    0, // user_error_fn
    0  // user_warning_fn
    );

  png_infop info_ptr = png_create_info_struct(png_ptr);
  png_infop end_info = png_create_info_struct(png_ptr);
  png_init_io(png_ptr, file);

  png_read_info(png_ptr, info_ptr);
  uint w         = png_get_image_width(png_ptr,info_ptr);
  uint h         = png_get_image_height(png_ptr,info_ptr);
  int ncomp      = png_get_channels(png_ptr,info_ptr);
  int bit_depth  = png_get_bit_depth(png_ptr,info_ptr);
  int color_type = png_get_color_type(png_ptr,info_ptr);
  if (w <= 0 || h <=0) {
    fprintf(stderr,"ImageFormat_PNG::load - corrupted PNG file (%s)",name);
    exit (-1);
  }
  if (bit_depth !=8 ) {
    fprintf(stderr,"ImageFormat_PNG::load - unsupported bit depth (%d bit in %s)",ncomp,name);
    exit (-1);
  }
  if (color_type == PNG_COLOR_TYPE_PALETTE) {
    fprintf(stderr,"ImageFormat_PNG::load - palette not supported (%s)",name);
    exit (-1);
  }

  vector<unsigned char> tmp;
  tmp.resize(h*w*ncomp);
  vector<png_bytep> rowptrs;
  rowptrs.resize(h);
  for (uint j=0;j<h;j++) {
    rowptrs[j]=(png_bytep)&(tmp[(j*w)*ncomp]);
  }
  png_read_image(png_ptr, &(rowptrs[0]));
  png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
  fclose(file);

  if (ncomp == 3) {
    ImageRGB *rgb=new ImageRGB(w,h);
    ForPixels(rgb,i,j) {
      for (uint c=0;c<3;c++)
        rgb->pixel(i,j)[c]=tmp[(i+j*w)*ncomp+c];
    }
    return (rgb);
  } else if (ncomp == 4) {
    ImageRGBA *rgba=new ImageRGBA(w,h);
    ForPixels(rgba,i,j) {
      for (uint c=0;c<4;c++)
        rgba->pixel(i,j)[c]=tmp[(i+j*w)*ncomp+c];
    }
    return (rgba);
  } else {
    fprintf(stderr,"ImageFormat_PNG::load - unsupported number of components (%d components in %s)",ncomp,name);
    exit (-1);
  }
}

//---------------------------------------------------------------------------


void NAMESPACE::ImageFormat_PNG::save(const char *,const NAMESPACE::Image *) const
{
    fprintf(stderr,"ImageFormat_PNG::save - NYI");
    exit (-1);
}

//---------------------------------------------------------------------------
